<?php
 class Uniteller_Model extends Samo_Acquiring { protected $moduleName = 'uniteller'; const PAY_TTL = 3600; public function __construct() { parent::__construct(); $this->checkBankParams(); $this->bankInc = $this->acquiringConfig('UNITELLER_SAMO_INC'); $this->bankAlias = $this->acquiringConfig('UNITELLER_SAMO_LOGIN'); } public function getFormAction() { if (!empty($this->acquiringConfig('UNITELLER_PRE_AUTH'))) { return $this->acquiringConfig('UNITELLER_API_URL') . '/preauth'; } else { return $this->acquiringConfig('UNITELLER_PAY_URL'); }; } public function getHiddenFields() { $return = [ 'Shop_IDP' => $this->acquiringConfig('UNITELLER_SHOP_ID'), 'Order_IDP' => '', 'Subtotal_P' => '', 'Lifetime' => self::PAY_TTL, 'PT_Code' => '', 'Signature' => '', 'URL_RETURN_OK' => '', 'URL_RETURN_NO' => '', 'URL_RETURN' => '', ]; if (empty($this->acquiringConfig('UNITELLER_PRE_AUTH')) && !empty($this->acquiringConfig('UNITELLER_FISCAL'))) { $return['Receipt'] = ''; $return['ReceiptSignature'] = ''; } if (!empty($this->acquiringConfig('UNITELLER_PRE_AUTH'))) { $return['Preauth'] = 1; } return $return; } public function getReceipt($params, WSPAY_Logger $logger) { $data = static::getPaymentInfoData($params['Order_IDP']); $email = $phone = ''; $lines = []; foreach ($data as $d) { if (!$email && $d['payee_email'] && Samo_Validate::email($d['payee_email'])) { $email = $d['payee_email']; } if (!$phone && $d['payee_phones']) { $phone = $d['payee_phones']; } $lines[] = [ 'name' => Samo_String::set($d['label']), 'price' => $d['price'], 'qty' => 1, 'sum' => $d['price'], 'vat' => $this->acquiringConfig('UNITELLER_FISCAL_TAX'), 'taxmode' => $this->acquiringConfig('UNITELLER_FISCAL_TAXATION_SYSTEM'), ]; } $receipt = [ 'customer' => [ 'phone' => $phone, 'email' => $email, 'id' => $this->defaults['CLAIM'], ], 'lines' => $lines, 'total' => $params['Subtotal_P'], ]; $json = json_encode($receipt); $logger->log($receipt, 'receipt'); return base64_encode($json); } public function makeFormData() { $this->checkBankParams(); $amount = $this->_amount; if (!$amount) { $this->_die($this->messages['UNITELLER_EMPTY_AMOUNT']); } $invoice = $this->getInvoice(); $inumber = $invoice['inumber']; $logger = $this->initLogger(__METHOD__, $inumber); $return = false; try { $return = [ 'Shop_IDP' => $this->acquiringConfig('UNITELLER_SHOP_ID'), 'Order_IDP' => $inumber, 'Subtotal_P' => $amount, 'MeanType' => '', 'EMoneyType' => '', 'Lifetime' => self::PAY_TTL, 'Customer_IDP' => '', 'Card_IDP' => '', 'IData' => '', 'PT_Code' => '', ]; $return['Signature'] = self::makeSignature($return, '&'); if (empty($this->acquiringConfig('UNITELLER_PRE_AUTH')) && !empty($this->acquiringConfig('UNITELLER_FISCAL'))) { $keys = [ 'Shop_IDP', 'Order_IDP', 'Subtotal_P', 'Receipt', ]; $return['Receipt'] = $this->getReceipt($return, $logger); $return['ReceiptSignature'] = self::makeReceiptSignature($return, $keys); } if (!empty($this->acquiringConfig('UNITELLER_PRE_AUTH'))) { $return['Preauth'] = 1; } $okUrl = $this->makeOkURL($invoice['inumber']); $failUrl = $this->makeFailURL($invoice['inumber']); $return['URL_RETURN_OK'] = $okUrl; $return['URL_RETURN_NO'] = $failUrl; $return['URL_RETURN'] = Samo_Request::scheme() . '://' . Samo_Request::host(); $logger->log($return, 'return'); if (!empty($this->acquiringConfig('UNITELLER_PRE_AUTH'))) { self::setHoldInvoiceByINumber($this->acquiringConfig('UNITELLER_SAMO_LOGIN'), $inumber); } $this->createCheckOrderStatusJob('return Uniteller_Model::processOrderStatus("' . $inumber . '", "");', self::PAY_TTL); } catch (Exception $e) { $logger->exception($e); } $logger->commit(); return $return; } public static function getPayStatus($inumber, $dataB64json = '') { $status = false; $billNumber = ''; $payStatus = ''; $logger = self::initLogger(__METHOD__, $inumber); try { $params = json_decode(base64_decode($dataB64json), true); $aPostData = [ 'Format' => 4, 'Shop_ID' => self::acquiringConfig('UNITELLER_SHOP_ID'), 'ShopOrderNumber' => $inumber, ]; $response = self::sendRequest(self::acquiringConfig('UNITELLER_API_URL') . '/results', $aPostData + self::getSecrets()); $logger->log($response, 'response_data'); $oXml = self::parseXML($response); if (empty($oXml->orders->order->status)) { throw new Samo_Exception('Wait user pay'); }; $payStatus = mb_strtolower((string)$oXml->orders->order->status); switch ($payStatus) { case 'authorized': $billNumber = (string)$oXml->orders->order->billnumber; $data = [ 'billnumber' => $billNumber, ]; self::setInvoiceExternalData($inumber, $data); self::confirmOrder(self::acquiringConfig('UNITELLER_SAMO_LOGIN'), $inumber); $status = true; break; case 'canceled': $status = true; break; default: $logger->commit(); throw new Samo_Exception('Unexpected payStatus : ' . var_export($payStatus, true)); }; } catch (Exception $e) { $logger->exception($e); $logger->commit(); throw new Samo_Exception($e->getMessage()); } $logger->commit(); $ret = [ 'status' => $status, 'info' => $payStatus, 'billnumber' => $billNumber ]; return $ret; } public static function processOrderStatus($inumber, $dataB64json = '') { try{ self::_getDB(); return self::getPayStatus($inumber, $dataB64json)['status']; } catch (Exception $e) { throw new Samo_Exception($e->getMessage()); } } public static function makeSignature($params, $exploder = '') { $arr = []; foreach ($params as $p) { $arr[] = md5($p || $p == '0' ? $p : ''); } $arr[] = md5(static::acquiringConfig('UNITELLER_PASSWORD')); $string = implode($exploder, $arr); return strtoupper(md5($string)); } public static function makeReceiptSignature($params, array $keys) { $arr = []; foreach ($keys as $key) { $value = isset($params[$key]) ? $params[$key] : ''; $arr[] = openssl_digest($value, 'sha256'); } $arr[] = openssl_digest(static::acquiringConfig('UNITELLER_PASSWORD'), 'sha256'); $string = implode('&', $arr); return strtoupper(openssl_digest($string, 'sha256')); } protected function _capture($invoice) { $orderId = $invoice['INumber']; $logger = $this->initLogger(__METHOD__, $orderId); $ret = false; try { $aReceipdParams = [ 'Order_IDP' => $orderId, 'Subtotal_P' => $invoice['RubSum'], ]; $aPostData = [ 'ShopID' => $this->acquiringConfig('UNITELLER_SHOP_ID'), 'OrderID' => $orderId, 'Subtotal' => $invoice['RubSum'], ]; $keys = [ 'ShopID', 'OrderID', 'Subtotal', 'Receipt', ]; $aPostData['Signature'] = self::makeSignature($aPostData); $aPostData['Receipt'] = $this->getReceipt($aReceipdParams, $logger); $aPostData['ReceiptSignature'] = self::makeReceiptSignature($aPostData, $keys); $responce = $this->sendRequest($this->acquiringConfig('UNITELLER_API_URL') . '/api/iaconfirm', $aPostData); $oXml = $this->parseXML($responce); if ($oXml->Result === 0) { $ret = true; }; $logger->log($aPostData, 'post_data'); $logger->log($responce, 'responce_data'); } catch (Exception $e) { $logger->exception($e); } $logger->commit(); return $ret; } protected function _reverse($invoice) { $inumber = $invoice['INumber']; $logger = $this->initLogger(__METHOD__, $inumber); $ret = false; try { $billNumber = (!empty($invoice['ExternalData']) ? $invoice['ExternalData']['billnumber'] : '' ); if (empty($billNumber)) { $billNumber = $this->getPayStatus($inumber)['billnumber']; } if (empty($billNumber)) { throw new Samo_Exception('billNumber empty'); } $aPostData = [ 'Shop_ID' => $this->acquiringConfig('UNITELLER_SHOP_ID'), 'Billnumber' => $billNumber, 'Format' => 3, ]; $responce = $this->sendRequest($this->acquiringConfig('UNITELLER_API_URL') . '/unblock', $aPostData + $this->getSecrets()); $oXml = $this->parseXML($responce); if ($oXml->Result === 0) { $ret = true; }; $logger->log($aPostData, 'post_data'); $logger->log($responce, 'responce_data'); } catch (Exception $e) { $logger->exception($e); } $logger->commit(); return $ret; } protected static function getSecrets() { $ret = [ 'Login' => self::acquiringConfig('UNITELLER_LOGIN'), 'Password' => self::acquiringConfig('UNITELLER_PASSWORD'), ]; return $ret; } protected static function sendRequest($url, $post) { $res = Samo_Curl::request($url, $post); if ($res === false) { throw new Samo_Exception(self::messages('ACQUIRING_RESPONSE_FAILED')); } return $res; } protected function checkBankParams() { $constants = ['PASSWORD', 'SHOP_ID', 'PAY_URL', 'SAMO_INC']; foreach ($constants as $suffix) { $name = 'UNITELLER_' . $suffix; if (!\defined($name) || !constant($name)) { $this->_die(Samo_EPlatez::messages('UNITELLER_PARAMS_NOT_SETS'), Pay_Variant_Model::UNAVAILABLE); break; } } if (\defined('UNITELLER_FISCAL') && UNITELLER_FISCAL) { if (!\defined('UNITELLER_FISCAL_TAX') || !\defined('UNITELLER_FISCAL_TAXATION_SYSTEM')) { $this->_die(Samo_EPlatez::messages('UNITELLER_PARAMS_NOT_SETS'), Pay_Variant_Model::UNAVAILABLE); } } return true; } } 